/*
 * This file is part of Jelie,
 * (c) 2002 Julien Pilet <julien.pilet@epfl.ch> and
 * Stephane Magnenat <stephane.magnenat@epfl.ch>
 *
 * Jelie is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * Jelie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*! \file
 * \brief debug messages handling
 *
 *  Error reporting and debug information handling macros. Allows selectionning
 *  what debug message to display from a list of choices (see DebugLevel).
 *
 * \author Julien Pilet
 */
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include "debug.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

int debugLevel=0;

void setDebugLevel(DebugLevel level) {
	debugLevel = level;
}

void addDebugLevel(DebugLevel l) {
	debugLevel |= l;
}

void removeDebugLevel(DebugLevel l) {
	debugLevel &= ~l;
}

void debugMessage(DebugLevel level, const char *szMessage, ...) {
        va_list arglist;

	if ((level & debugLevel) == 0) 
		return;
        va_start(arglist, szMessage);
#ifdef HAVE_VPRINTF
        vprintf( szMessage, arglist);
#else
	printf("(vprintf not available) %s", szMessage);
#endif
        va_end(arglist);

        //fprintf(stderr, " %s", str);
        //MessageBox(SYSW_GetHwnd(), str, "Warning", MB_ICONEXCLAMATION);
}

