/*
 * This file is part of Jelie,
 * (c) 2002 Julien Pilet <julien.pilet@epfl.ch> and
 * Stephane Magnenat <stephane.magnenat@epfl.ch>
 *
 * Jelie is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * Jelie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*! \file 
 * \brief debug messages handling
 *
 *  Error reporting and debug information handling macros.
 * \author Julien Pilet
 */

#ifndef _DEBUG_H_
#define _DEBUG_H_

//! List of all the debugging options.
typedef enum {
	//! prints debug messages about the gdb connection and protocol
	GDB_REMOTE_DEBUG=0x2, 

	//! prints debug messages about scripting.
	CMD_SCRIPT_DEBUG=0x4,
}DebugLevel ;


//! replace the current debugging level by the a new one.
void setDebugLevel(DebugLevel level);

//! add the debug level passed in parameter to the current debug level.
void addDebugLevel(DebugLevel level);

//! remove a debug level from the current debug level.
void removeDebugLevel(DebugLevel level);

//! print a debug message if it's level is selected.
void debugMessage(DebugLevel level, const char *szMessage, ...);

#endif

